<!DOCTYPE html>
<html class="homepage">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>{{ site.data.project.tagline }} - {{ site.data.project.title }}</title>
    <meta name="description" content="{{ site.data.project.description }}">
    <link rel="icon" type="image/x-icon" href="//theme.thephpleague.com/img/favicon.ico" />
    <link rel="apple-touch-icon-precomposed" href="//theme.thephpleague.com/img/apple-touch-icon-precomposed.png">
    <link rel="stylesheet" href="//theme.thephpleague.com/css/all.css?2">
    <link rel="stylesheet" href="/homepage.css?8">
    <script async src="https://www.googletagmanager.com/gtag/js?id={{  site.data.project.google_analytics_tracking_id }}"></script>
    <script>
        window.dataLayer = window.dataLayer || [];
        function gtag() { dataLayer.push(arguments);}
        gtag('js', new Date());
        gtag('config', '{{  site.data.project.google_analytics_tracking_id }}');
    </script>
</head>
<body>

<header>
    <div class="inner-content">
      <h1>{{ site.data.project.title }}</h1>
      <h2>{{ site.data.project.tagline }}</h2>
    </div>
    <p class="footnote">{{ site.data.project.support }}</p>
</header>

<aside class="sponsors hide">
    <p>Love this package ? <a href="https://github.com/sponsors/{{ site.data.project.author.github_account }}">Support its development!</a></p>
</aside>

<main>

  <div class="features">
    <div class="inner-content">
        <h1>Intuitive</h1>
        {% include building-block.md %}
    </div>
  </div>

    <div class="highlights">
        <div class="inner-content">
            <div class="column one">
                <h1>Highlights</h1>
                <div class="description">
                    <p>This suite of packages provides intuitive classes to parse, validate, format and manipulate URIs and their components. It is built to enable working with
                        any kind of RFC3986 compliant URI and also follow closely the latest WHATWG Living standard specification.</p>
                    <p>It provides an enhanced replacement for PHP's <code>parse_url</code>, <code>http_build_query</code>, and PECL's <code>http_build_url</code> functions.</p>
                </div>
            </div>
            <div class="column two">
                <ol>
                    <li><p>Simple and extensible API</p></li>
                    <li><p><a href="http://tools.ietf.org/html/rfc3986">RFC3986</a> compliant</p></li>
                    <li><p>follows <a href="https://url.spec.whatwg.org/">URL Living Standard</a> specifications</p></li>
                    <li><p>Implements <a href="http://www.php-fig.org/psr/psr-7/">PSR-7</a> <code>UriInterface</code> interface</p></li>
                    <li><p>Framework-agnostic</p></li>
                </ol>
            </div>
        </div>
    </div>

    <div class="documentation">
        <div class="inner-content">
            <h1>Packages</h1>
            <div class="packages-list">
                <h1>building blocks</h1>
                <dl>
                    <dt><a href="/interfaces/7.0/">uri-interfaces</a></dt>
                    <dd>Represents the backbone of all the packages. It contains contracts and tools that can be used independently of any URI package.</dd>
                </dl>
                <dl>
                    <dt><a href="/uri/7.0/">uri</a></dt>
                    <dd>The main package. Its goal is to ease the generation and manipulation of URI objects, independently of their scheme.</dd>
                </dl>
                <dl>
                    <dt><a href="/components/7.0/">uri-components</a></dt>
                    <dd>Complements the URI package, providing an object-oriented way to access and manipulate each component value of the underlying URI.</dd>
                </dl>
            </div>
        </div>
    </div>

  <div class="questions">
    <div class="inner-content">
      <h1>Questions?</h1>
      <p>The packages were created by {{ site.data.project.author.name }}.<br>
          Find him
          on Mastodon at <a href="{{ site.data.project.author.mastodon.url}}">{{ site.data.project.author.mastodon.account}}</a> or
          on Twitter at <a href="https://twitter.com/{{ site.data.project.author.twitter_account }}">@{{ site.data.project.author.twitter_account }}</a>
      </p>
    </div>
  </div>
</main>

<footer>
    <span>&copy; Copyright <a href="//thephpleague.com">The League of Extraordinary Packages</a>.</span>
    <span>Site design by <a href="//reinink.ca">Jonathan Reinink</a> and <a href="//nyamsprod.com">Ignace Nyamagana Butera</a>.</span>
</footer>
<script src="/custom.js?3"></script>
</body>
</html>
